// An ADD comment will be used to indicate variables and code added
// to the shader to increase functionality.
// The CHANGE comment will show changes made to demo shader code from 
// previous version.

// ADD
// PREFACE
// We scale, rotate, and translate our collection of vertices (represented as 
// homogeneous coordinates) through multiplication by 4x4 matrices. There are 
// three such matrices of general relevance: model, view, and projection. Check 
// a reference book or search for these terms for more information.
//
// The model matrix transforms (scales, rotates, translates) the original 
// vertex coordinates of the geometry (cube, sphere, etc), which might have been 
// constructed around the origin, into the position we want in world (or data) 
// space. For instance, you might build the vertices and triangles to represent 
// a sphere around the origin, scale it to an aesthetically appealing size, then 
// translate it to a position to represent the physical location of some point 
// of interest. 
// 
// The view matrix can be applied to the results of the above to transform the 
// desired (often all of the) elements in world space into the space viewed by 
// the camera - in OpenGL usually, -1 to 1 in the x and y directions and 0 to 1 
// for z. The x,y limits are defined by the OpenGL window and the z limits are 
// cutoffs (clipping planes) defined within OpenGL so that neither extremely 
// close objects nor extremely far objects are plotted. An internet search for 
// "frustum" might be useful. Note, too, that you can have wide, short windows 
// or tall, narrow ones that are still represented by the above x,y limits. So, 
// we will have to deal with the aspect ratio of the window to keep the original 
// geometry from being distorted. We did not do that for the earlier triangles, 
// because we need a shader-based transformation to do that, which we have not 
// had until now.
// 
// Finally, a projection is applied to these coordinates to simulate the 
// projection of the data onto the imaging plane of a camera (or eye) such that 
// parallel lines (e.g., train tracks) converge in the distance. This is the 
// perspective transformation. Omitting this step loses the appearance of 
// perspective, but makes distances between points on the screen comparable 
// regardless of distance from the viewer, as might be useful to an architect. 
//
// All of the above are cumulative in that they can be represented by a matrix
// constructed by the sequential multiplication the individual matrices
// into a single, final matrix. E.g., model X view x projection = total.
// Furthermore, one might mention that the model and view matrices can similarly
// be constructed by the multiplication of individual matrices representing
// individual types of transformations: e.g., scale X rotation X translation = 
// model. NOTE: the result is order dependent - multiplication by a rotation
// matrix before multiplying by a translation matrix does not yield the same
// results as the reverse order. Matrix multiplication IS NOT commutative.

#version 150 core

in vec3 position;

in vec3 color;

// ADD
// We have position and color, and here we add the 4x4 combined
// model X view X projection, mvpMx. The relationship between these local 
// variables and the data are set in the OpenGL init(...) method provided by the 
// implementing Java class.
// The "uniform" qualifier means the variable will not change during the 
// processing of the primitive specified in the program, e.g., vertices of the 
// triangle. See the program code to find out how this matrix is constructed.
uniform mat4 mvpMx;

// CHANGE
// These are the user variables in which values are passed out of the shader.
// We now send out the vertex color plus alpha (transparency) value as a vec4.
out vec4 Color;

void main()
{
    // CHANGE
    // We transform the coordinates by the matrix.
    gl_Position = mvpMx * vec4(position, 1.0);

    // CHANGE
    // Not necessary for now, e.g., not animation related, but moves some of the 
    // color manipulation out of the fragment shader. We will do more later.
    // We now pass the color as rbga, where a indicates the alpha (or 
    // transparency) value 1=opaque. OpenGL will handle interpolating areas 
    // between vertices.
    Color = vec4(color, 1.0f);
}
